/* -*- C++ -*- *************************************************************
 *
 *  model.cpp, Tue Mar 5 2002
 *  Copyright (C) 2002 Soloviev Alexei <solovjev@cv.jinr.ru>, Alexei Stadnik
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "model.h"
#include "rc.h"

#include <complex>
using namespace std;

Model::Model(const vector<Parameter> & rhs, const string & name)
  : fParm(rhs), fName(name)
{}

Model::~Model() {}

/** Number of parameters */
size_type Model::size() const 
{
  return fParm.size();
}

/** Access to parameter no. index (read) */
const double & Model::operator[](size_type index) const 
{
  return fParm[index].value();
}

/** Access to parameter no. index (write) */
double & Model::operator[](size_type index) 
{
  return fParm[index].value();
}

/** Access to error of parameter no. index (read) */
const double & Model::error(size_type index) const
{
  return fParm[index].error();
}

/** Access to min value of parameter no. index (read) */
const double & Model::min(size_type index) const
{
  return fParm[index].min();
}

/** Access to max value of parameter no. index (read) */
const double & Model::max(size_type index) const
{
  return fParm[index].max();
}

/** Access to error of parameter no. index (write) */
double & Model::error(size_type index) 
{
  return fParm[index].error();
}

/** Name of parameter no. index */
const string & Model::name(size_type index) const 
{
  return fParm[index].name();
}

/** Model name */
const string & Model::name() const 
{
  return fName;
}

/** Read model parameters from rc-file */
void Model::rc_read()
{
  ircfile rcin;
  if ( rcin ) {
    rc_section<complex<double> > modrc ( name() );
    for ( int i = 0; i < size(); ++i ) {
      modrc.insert ( name(i), 
		     complex<double> ( fParm[i].value(), fParm[i].error() ) );
    }
    rcin >> modrc;
    for ( int i = 0; i < size(); ++i ) {
      fParm[i].value() = modrc[name(i)].real();
      fParm[i].error() = modrc[name(i)].imag();
    }
  }
}

/** Write model parameters to rc-file */
void Model::rc_write() const
{
  rc_section<complex<double> > modrc ( name() );
  for ( int i = 0; i < size(); ++i ) {
    modrc.insert ( name(i), 
		   complex<double> ( fParm[i].value(), fParm[i].error() ),
		   "("+name(i)+" value, "+name(i)+" uncertainty)" );
  }
  ircfile rcin;
  if ( rcin ) { // rc-file exists
    rcin.backup();
    rcfile rc;
    rc.rm ( modrc );
    rc << modrc;
  } else { // first writing to rc-file
    orcfile rcout;
    rcout << modrc;
  }
}
