/* -*- C++ -*- *************************************************************
 *
 *  parameter.h, Tue Mar 5 2002
 *  Copyright (C) 2002 Soloviev Alexei <solovjev@cv.jinr.ru>, Alexei Stadnik
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PARAMETER_H
#define PARAMETER_H

#include <string>
using namespace std;

class Parameter {
public: 
  Parameter ( const string & name,
	      const double & value,
	      const double & error );
  Parameter ( const string & name,
	      const double & value,
	      const double & error,
	      const double & min,
	      const double & max );
  ~Parameter ();

public:
  /** Get parameter description */
  const std::string & name() const;

  /** Access to parameter value (read) */
  const double & value() const;
  /** Access to parameter value (write) */
  double & value();

  /** Access to parameter error (read) */
  const double & error() const;
  /** Access to parameter error (write) */
  double & error();

  /** Access to parameter min value (read) */
  const double & min() const;
  /** Access to parameter max value (read) */
  const double & max() const;

private: // Private attributes
  /** Parameter description */
  string fName;
  /** Parameter current value */
  double fValue;
  /** Parameter current error */
  double fError;
  /** Parameter minimum value */
  double fMin;
  /** Parameter maximum value */
  double fMax;
};

#endif
