/***************************************************************************
                          sansmodel.h  -  description
                             -------------------
    begin                : Mon Jan 13 2003
    copyright            : (C) 2003 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef YuMOMODEL_H
#define YuMOMODEL_H

#include "sansmodel.h"

#include <map>

/** SANS models used for YuMO spectrometer
  * @author Alexei Soloviev
  */

class YuMOModel : public SANSModel {
 public:
  YuMOModel ( const SANSModel & rhs );
  virtual ~YuMOModel();

 public:
  /** Theoretical function */
  double operator() ( const double & x ) const;

 public:
  /** To construct model over comand line parameters */
  static const Dictionary<YuMOModel> & dictionary();

 public:
  /** Ball constructor */
  static const YuMOModel ball();
  /** Ellipsoid-2 constructor */
  static const YuMOModel ellipsoid2();
  /** Ellipsoid-3 constructor */
  static const YuMOModel ellipsoid3();
  /** Cylinder constructor */
  static const YuMOModel cylinder();
  /** Cylinder-2 constructor */
  static const YuMOModel cylinder2();
  /** Parallelepiped constructor */
  static const YuMOModel parallelepiped();
  /** Spherical shell constructor */
  static const YuMOModel sphericalShell();
  /** Ellipsoid-2 shell constructor */
  static const YuMOModel ellipsoid2Shell();
  /** Ellipsoid-3 shell constructor */
  static const YuMOModel ellipsoid3Shell();

 public:
  /** Init Q-resolution dependency */
  bool initResolution ( const double * Q_ptr,
			const double * dQ_ptr, 
			const size_type n ) const;

 private: // Private attributes
  /** Shared variables */
  mutable struct {
    double Q, dQ;
  } fMemory;
  /** Q-resolution dependency */
  mutable map<double,double> fResolution;

 private: // Private methods
  /** Function to be integrated */
  double sans_theor_fun ( const double x ) const;
};

#endif
