/* -*- C++ -*- *************************************************************
 *
 *   rc_file.h, Mon Oct 27 2003
 *   Copyright (C) 2003 Alexei Soloviev <solovjev@cv.jinr.ru>
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RC_FILE_H
#define RC_FILE_H

#include "rc_section.h"

#include <fstream>
#include <string>
using namespace std;

class rc_file;
class rc_file_i;
class rc_file_o;

typedef rc_file rcfile;
typedef rc_file_i ircfile;
typedef rc_file_o orcfile;

class rc_file : public fstream {
public:
  /** C-tor: using argument */
  rc_file ( const string & __name, ios::openmode __mode = ios::in|ios::out );
#ifdef HAVE_CONFIG_H
  /** C-tor: using definition from <config.h> */
  rc_file ( ios::openmode __mode = ios::in|ios::out );
#endif

  virtual ~rc_file();

  /** Return rc-file full name */
  const string & name() const { return _name; }
  /** Return rc-file creation date */
  const string & date() const { return _date; }
  /** Return rc-file creation time */
  const string & time() const { return _time; }

  /** Make rc-file backup */
  void backup();
  /** Remove rc-section from rc-file */
  void rm ( const string & rcsname );
  /** Remove rc-section from rc-file */
  template <class T>
  void rm ( const rc_section<T> & rcs ) { rm ( rcs.name() ); }

protected:
  static const string _home;
  const string _name;
  ios::openmode _mode;
  string _date, _time;
  string _header;

protected:
  void init();
};

class rc_file_i : public rc_file {
public:
  /** C-tor: using argument */
  explicit rc_file_i ( const string & __name );
#ifdef HAVE_CONFIG_H
  /** C-tor: using definition from <config.h> */
  rc_file_i();
#endif

  virtual ~rc_file_i();
};

class rc_file_o : public rc_file {
public:
  /** C-tor: using argument */
  explicit rc_file_o ( const string & __name );
#ifdef HAVE_CONFIG_H
  /** C-tor: using definition from <config.h> */
  rc_file_o();
#endif

  virtual ~rc_file_o();
};

#endif // RC_FILE_H
