/* -*- C++ -*- *************************************************************
 *
 *   rc_section.h, Mon Oct 27 2003
 *   Copyright (C) 2003 Alexei Soloviev <solovjev@cv.jinr.ru>
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RC_SECTION_H
#define RC_SECTION_H

#include "rc_sign.h"

#include <map>
#include <string>
#include <iostream>
#include <iterator>
#include <iomanip>
#include <algorithm>
using namespace std;

template <class T>
class rc_section : private map<string,pair<T,string> > {
private:
  typedef map<string,pair<T,string> > _base_type;

public:
  typedef T data_type;
  typedef pair<T,string> value_type;

  typedef data_type& reference;
  typedef const data_type& const_reference;

  typedef typename _base_type::iterator iterator;
  typedef typename _base_type::const_iterator const_iterator;
  typedef typename _base_type::reverse_iterator reverse_iterator;
  typedef typename _base_type::const_reverse_iterator const_reverse_iterator;

  typedef typename _base_type::size_type size_type;

public:
  explicit rc_section ( const string & name, 
			const string & description = string(), 
			const string & comments = string() ) :
    _base_type(), 
    __name(name), __description(description), __comments(comments) {}
  virtual ~rc_section() {}

  void insert ( const string & key, 
		const T & value, 
		const string & description = string() )
  { _base_type::insert(make_pair(key,make_pair(value,description))); }

  const_iterator find ( const string & __x ) const { return _base_type::find(__x); }
  iterator find ( const string & __x ) { return _base_type::find(__x); }

  const_reference operator[] ( const string & __x ) const;
  reference operator[] ( const string & __x );

  const_iterator begin() const { return _base_type::begin(); }
  iterator begin() { return _base_type::begin(); }
  const_iterator end() const { return _base_type::end(); }
  iterator end() { return _base_type::end(); }

  bool empty() const { return _base_type::empty(); }
  size_type size() const { return _base_type::size(); }

private:
  const string __name;
  const string __description;
  string __comments;

public:
  const string & name() const { return __name; }
  const string & description() const { return __description; }
  const string & comments() const { return __comments; }
  string & comments() { return __comments; }
};

template <class T>
ostream & operator<< ( ostream & out, const rc_section<T> & rcs );

template <class T>
istream & operator>> ( istream & in, rc_section<T> & rcs );

/***************************************************************************
  Implementations
 ***************************************************************************/

template <class T>
inline
const T & rc_section<T>::operator[] ( const string & __x ) const
{
  const_iterator i = find(__x);
  if ( i == end() )
    cerr<<"rc_section: \""<<name()<<"\" has no field named \""<<__x<<"\""<<endl;
  return i->second.first;
}

template <class T>
inline
T & rc_section<T>::operator[] ( const string & __x )
{
  iterator i = find(__x);
  if ( i == end() )
    cerr<<"rc_section: \""<<name()<<"\" has no field named \""<<__x<<"\""<<endl;
  return i->second.first;
}

template <class T>
ostream & operator<< ( ostream & out, const rc_section<T> & rcs )
{
  out << rcs.name() << endl;
  if ( !rcs.description().empty() ) 
    out << comment_sign() << rcs.description() << endl;
  out << begin_sign() << endl;
  if ( !rcs.comments().empty() ) 
    out << comment_sign() << rcs.comments() << endl;

  out.setf ( ios::left,ios::adjustfield );
  typename rc_section<T>::const_iterator p;
  unsigned int width = 0;
  for ( p = rcs.begin(); p != rcs.end(); ++p  ) {
    width = max ( width, static_cast<unsigned int>(p->first.size()) );
  }
  width += 2;
  for ( p = rcs.begin(); p != rcs.end(); ++p  ) {
    out << setw(width) << p->first.c_str();
    out << p->second.first << "\t";
    if ( !p->second.second.empty() ) 
      out << comment_sign() << p->second.second;
    out << endl;
  }

  out << end_sign() << endl;

  return out;
}

template <class T>
istream & operator>> ( istream & in, rc_section<T> & rcs )
{
  string buff;
  do {
    char c;
    while ( in.get ( c ) && c != '\n' ) buff += c; // "getline"
  } while ( in && buff.find ( rcs.name() ) == buff.npos );
  istream_iterator<string> cur ( in ), eof;
  cur = find ( cur, eof, begin_sign() );
  string key;
  while ( in >> key && key != end_sign() && key != comment_sign() ) {
    if ( rcs.find ( key ) != rcs.end() ) {
      ( in >> rcs[key] ).ignore ( 1024, '\n' );
    }
  }
  return in;
}

#endif // RC_SECTION_H
