/* -*- C++ -*- *************************************************************
 *
 *   usage.cpp, Fri Feb 27 2004
 *   Copyright (C) 2004 Soloviev Alexei <solovjev@cv.jinr.ru>
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "usage.h"

// Fitter long options for getopt
struct option long_options[] = {
  {"std", 1, 0, 0},
  {"sans", 1, 0, 0},
  {"yumo", 1, 0, 0},
  {"robust", 0, 0, 'r'},
  {"verbose", 2, 0, 'v'},
  {"strategy", 1, 0, 's'},
  {"range", 1, 0, 'R'},
  {"output", 1, 0, 'o'},
  {"header", 1, 0, 'H'},
  {"rc-read", 0, 0, '>'},
  {"rc-write", 0, 0, '<'},
  {"help", 0, 0, 'h'},
  {"version", 0, 0, 'V'},
  {0, 0, 0, 0}
};

// Fitter options handbook
Book & book()
{
  static Book fBook;

  if ( fBook.empty() ) {

    static Section fModel;
    if ( fModel.empty() ) {
      fModel["std"] = "Standard models";
      fModel["sans"] = "SANS models";
      fModel["yumo"] = "SANS models for YuMO spectrometer";
    }
    fBook["Model"] = fModel;

    static Section fProcessing;
    if ( fProcessing.empty() ) {
      fProcessing["robust"] = "Robust fitting [no]";
      fProcessing["verbose"] = "Output MINUIT processing information [no]";
      fProcessing["strategy"] = "Change MINUIT strategy [MINIMIZE]";
      fProcessing["range"] = "Fitting in range x1:x2 [none]";
      fProcessing["SectionArgument"] = "val";
    }
    fBook["Processing"] = fProcessing;

    static Section fFile;
    if ( fFile.empty() ) {
      fFile["output"] = "Output data file [none]";
      fFile["header"] = "Output results file [standard output]";
      fFile["rc-read"] = "Read rc-file [no]";
      fFile["rc-write"] = "Write rc-file [no]";
    }
    fBook["File"] = fFile;

    static Section fHelp;
    if ( fHelp.empty() ) {
      fHelp["help"] = "Display this help and exit";
      fHelp["version"] = "Display version information and exit";
    }
    fBook["Help"] = fHelp;

  }
  return fBook;
}

#include <string>
using namespace std;

// short options for getopt
const char * short_options()
{
  option_index = 0;
  static string optstring;
  if ( optstring.empty() ) {
    option * optptr = long_options;
    while ( optptr->name != 0 ) {
      if ( optptr->val && isalpha ( optptr->val ) ) {
	optstring += optptr->val;
	if ( optptr->has_arg == 1 ) optstring += ':';
	if ( optptr->has_arg == 2 ) optstring += '?';
      }
      ++optptr;
    }
  }
  return optstring.c_str();
}

// option index for getopt
int option_index;

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <iomanip>
#include <algorithm>
#include <functional>
#include <cctype>
using namespace std;

// general usage
void usage ( ostream & out )
{
#ifdef PACKAGE
  out << "Usage: " << PACKAGE << " [options] <file>" << endl;
#endif

  out << "Options: [defaults in brackets after descriptions]" << endl;

  static const int opt_width = 24;
  out.setf(ios::left,ios::adjustfield);

  Book::const_iterator sect = book().begin();
  while ( sect != book().end() ) {
    out << sect->first << " options:" << endl;
    option * optptr = long_options;
    while ( optptr->name != 0 ) {
      if ( book()[sect->first].find( optptr->name ) != 
	   book()[sect->first].end() ) {
	string optstr = "  ";
	if ( optptr->val && isalpha ( optptr->val ) ) {
	  ( ( optstr += "-" ) += optptr->val ) += ", ";
	}
	( optstr += "--" ) += optptr->name;
	if ( optptr->has_arg ) {
	  string arg = sect->first;
	  if ( book()[sect->first].find( "SectionArgument" ) !=
	       book()[sect->first].end() ) {
	    arg = book()[sect->first]["SectionArgument"];
	  }
	  transform ( arg.begin(), arg.end(), arg.begin(),
		      ptr_fun ( ::tolower ) );
          if ( optptr->has_arg == 2 ) 
	    ( ( optstr += " [" ) += arg ) += "] ";
          else 
	    ( ( optstr += " <" ) += arg ) += "> ";
	}
	out << setw(opt_width) << optstr.c_str();
	out << book()[sect->first][optptr->name] << endl;
      }
      ++optptr;
    }
    ++sect;
  }
}
